==> privater-ddns_1.1.tar.bz2: include/functions.php <==
<?php
// Authentifizierung
// ----------------------------------------------------------------------------
function authenticate($user, $pass, $userdb) {
   if ( array_key_exists($user, $userdb) ) {
      // User OK
      write_log("Auth: User '".$user."' OK");
      if ( $userdb[$user] == $pass) {
         // Passwort OK
         write_debug("Auth: Pass for '".$user."' OK");
         return true;
      } else {
         // Passwort falsch
         write_debug("Auth: Password for '".$user."' wrong");
         return false;
      }
   } else {
      // Benutzer unbekannt
      write_log("Auth: User '".$user."' empty/unknown");
      return false;
   }
}

// Gültige IPv4-Adresse
// ----------------------------------------------------------------------------
function is_ipv4($ipv4) {
   // OK ist 250-255 oder 200-249 oder 100-199 oder 1-/2-stellige Zahl
   $num = "(25[0-5]|2[0-4]\d|[01]?\d\d|\d)";
   if (preg_match("/^($num)\\.($num)\\.($num)\\.($num)$/", $ipv4)) {
      return true;
   } else {
      return false;
   }
}

// Gültige IPv6-Adresse 
// ----------------------------------------------------------------------------
function is_ipv6($ipv6) {
   // Möglichkeiten
   $pattern1 = "([A-Fa-f0-9]{0,4}:){7}[A-Fa-f0-9]{0,4}";
   $pattern2 = "[A-Fa-f0-9]{0,4}::([A-Fa-f0-9]{0,4}:){0,5}[A-Fa-f0-9]{0,4}";
   $pattern3 = "([A-Fa-f0-9]{0,4}:){2}:([A-Fa-f0-9]{0,4}:){0,4}[A-Fa-f0-9]{0,4}";
   $pattern4 = "([A-Fa-f0-9]{0,4}:){3}:([A-Fa-f0-9]{0,4}:){0,3}[A-Fa-f0-9]{0,4}";
   $pattern5 = "([A-Fa-f0-9]{0,4}:){4}:([A-Fa-f0-9]{0,4}:){0,2}[A-Fa-f0-9]{0,4}";
   $pattern6 = "([A-Fa-f0-9]{0,4}:){5}:([A-Fa-f0-9]{0,4}:){0,1}[A-Fa-f0-9]{0,4}";
   $pattern7 = "([A-Fa-f0-9]{0,4}:){6}:[A-Fa-f0-9]{0,4}";
   if (preg_match("/^($pattern1)$|^($pattern2)$|^($pattern3)$|^($pattern4)$|^($pattern5)$|^($pattern6)$|^($pattern7)$/", $ipv6)) {
      return true;
   } else {
      return false;
   }
}

// IP in Datei schreiben
// ----------------------------------------------------------------------------
function write_data($file, $string) {
   if ( ($file == "") || ($string == "") ) {
      // Kein Dateiname oder Inhalt zum Schreiben
      write_log("Write file: Filename/content missing");
      return false;
   } else {
      if ( is_writable(DATA_FOLDER) ) {
         // Ordner beschreibbar
         $write_result = file_put_contents( DATA_FOLDER."/".$file.".ip", $string."\n" );
         if ( $write_result === false ) {
            // Schreiben fehlgeschlagen
            write_log("Write file '".$file.".ip' failed");
            return false;
         } else {
            // Schreiben OK, N Bytes geschrieben
            write_debug("Write file '".$file.".ip' OK");
            return true;
         }
      } else {
         // Ordner nicht beschreibbar
         write_log("Write: Folder '".DATA_FOLDER."' not writable");
         return false;
      }
   }
}

// IP aus Datei lesen
// ----------------------------------------------------------------------------
function read_data($file) {
   if ( $file == "" ) {
      // Kein Dateiname angegeben
      write_log("Read file: Filename missing");
      return false;
   } else {
      if ( is_readable(DATA_FOLDER."/".$file.".ip") ) {
         // Inhalt lesen
         $read_result = file_get_contents( DATA_FOLDER."/".$file.".ip" );
         write_debug("Read file '".$file.".ip' OK");
         return trim(str_replace("\n","", $read_result));
      } else {
         // Ordner nicht lesbar/vorhanden
         write_log("Read file: File '".$file.".ip' not readable");
         return false;
      }
   }
}

// Zeitstempel
// ----------------------------------------------------------------------------
function simple_timestamp() {
   // 2015-03-04_05:53:20
   $my_timestamp = date("Y-m-d_H:i:s_T");
   return $my_timestamp;
}

// Protokollierung (Ereignisse)
// ----------------------------------------------------------------------------
function write_log($string) {
   $write_result = file_put_contents(LOG_FILE, TIMESTAMP." ".pad_ipv4(CLIENT_IP)." ".$string."\n", FILE_APPEND);
   if ( $write_result === false ) {
      // Schreiben fehlgeschlagen
      return "Error";
   } else {
      // Schreiben OK, N Bytes geschrieben
      return "OK, ".$write_result." bytes written";            
   }
}

// Protokollierung (Debugging)
// ----------------------------------------------------------------------------
function write_debug($string) {
   if ( DEBUG === true ) {
      // Erst normales Logging
      write_log($string);
      // Und wenn Debugging aktiv
      $write_result = file_put_contents(LOG_FILE_DBG, TIMESTAMP." ".pad_ipv4(CLIENT_IP)." ".$string."\n", FILE_APPEND);
      if ( $write_result === false ) {
         // Schreiben fehlgeschlagen
         return "Error";
      } else {
         // Schreiben OK, N Bytes geschrieben
         return "OK, ".$write_result." bytes written";
      }
   }
}

// DNS-Aktualisierung
// ----------------------------------------------------------------------------
function update_dns($host, $ip) {
   if ( ( $host == "" ) || ( $ip == "" ) ) {
      // Hostname oder IP fehlt
      write_log("Nsupdate: Hostname or IP missing");
      return false;
   } else {
      // Server, Key und Zone
      $ddns_update_command  = "server ".DNS_SERVER."\n";
      $ddns_update_command .= "key ".KEY_NAME." ".KEY_VALUE."\n";
      $ddns_update_command .= "zone ".DDNS_ZONE.".\n";
      // A, AAAA und TXT entfernen
      $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". A\n";
      $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". AAAA\n";
      $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". TXT\n";
      // A (IPv4) / AAAA (IPv6) und TXT hinzufügen
      $ddns_update_command .= "update add ".$host.".".DDNS_ZONE.". ".TTL_SECONDS." IN ".DNS_RTYPE." ".$ip."\n";
      $ddns_update_command .= "update add ".$host.".".DDNS_ZONE.". ".TTL_SECONDS." IN TXT \"".TIMESTAMP."\"\n";
      $ddns_update_command .= "send\n";
      // Debug
      write_debug("Nsupdate running command '".$ddns_update_command."'");
      // Ausführen
      $cmdout = shell_exec("echo \"".$ddns_update_command. "\" | ".NSUPDATE." 2>&1; echo $?");
      // Debug
      write_debug("Nsupdate result '".print_error_status($cmdout)."'");
      return true;
   }
}

// Zeitstempel in lesbares Format 
// ----------------------------------------------------------------------------
function timestamp_to_readable($timestamp) {
   // 20150304-055820
   return date("Ymd-His", $timestamp);
}

// Fehler-Anzeige
// ----------------------------------------------------------------------------
function print_error_status($errorcode) {
   $errorcode = trim(str_replace("\n", "", $errorcode));
   if ( $errorcode == "0" ) {
      // Erfolg
      return "OK";
   } else {
      // Fehler
      return "Error '#".$errorcode."'";
   }
}

// IPv4-Adresse füllen auf xxx.xxx.xxx.xxx
// ----------------------------------------------------------------------------
function pad_ipv4($address) {
   // Zerschneiden
   $ip_parts = explode(".", $address);
   // Füllen
   $ip_parts[0] = str_pad($ip_parts[0], 3, "0", STR_PAD_LEFT);
   $ip_parts[1] = str_pad($ip_parts[1], 3, "0", STR_PAD_LEFT);
   $ip_parts[2] = str_pad($ip_parts[2], 3, "0", STR_PAD_LEFT);
   $ip_parts[3] = str_pad($ip_parts[3], 3, "0", STR_PAD_LEFT);
   // Zusammenkleben
   $address = implode(".", $ip_parts);
   // Return
   return $address;
}

// Bool zu String
// ----------------------------------------------------------------------------
function strbool($value) {
   return $value ? 'OK' : 'Not OK';
}
?>

==> privater-ddns_2.0.tar.bz2: include/functions.php <==
<?php
// Authentifizierung
// ----------------------------------------------------------------------------
function authenticate($user, $pass, $userdb) {
   if ( array_key_exists($user, $userdb) ) {
      // User OK
      write_log("Auth: User '".$user."' OK");
      if ( $userdb[$user] == $pass) {
         // Passwort OK
         write_debug("Auth: Pass for '".$user."' OK");
         return true;
      } else {
         // Passwort falsch
         write_debug("Auth: Password for '".$user."' wrong");
         return false;
      }
   } else {
      // Benutzer unbekannt
      write_log("Auth: User '".$user."' empty/unknown");
      return false;
   }
}

// Gültige IPv4-Adresse
// ----------------------------------------------------------------------------
function is_ipv4($ipv4) {
   // OK ist 250-255 oder 200-249 oder 100-199 oder 1-/2-stellige Zahl
   $num = "(25[0-5]|2[0-4]\d|[01]?\d\d|\d)";
   if (preg_match("/^($num)\\.($num)\\.($num)\\.($num)$/", $ipv4)) {
      return true;
   } else {
      return false;
   }
}

// Gültige IPv6-Adresse 
// ----------------------------------------------------------------------------
function is_ipv6($ipv6) {
   // Möglichkeiten
   $pattern1 = "([A-Fa-f0-9]{0,4}:){7}[A-Fa-f0-9]{0,4}";
   $pattern2 = "[A-Fa-f0-9]{0,4}::([A-Fa-f0-9]{0,4}:){0,5}[A-Fa-f0-9]{0,4}";
   $pattern3 = "([A-Fa-f0-9]{0,4}:){2}:([A-Fa-f0-9]{0,4}:){0,4}[A-Fa-f0-9]{0,4}";
   $pattern4 = "([A-Fa-f0-9]{0,4}:){3}:([A-Fa-f0-9]{0,4}:){0,3}[A-Fa-f0-9]{0,4}";
   $pattern5 = "([A-Fa-f0-9]{0,4}:){4}:([A-Fa-f0-9]{0,4}:){0,2}[A-Fa-f0-9]{0,4}";
   $pattern6 = "([A-Fa-f0-9]{0,4}:){5}:([A-Fa-f0-9]{0,4}:){0,1}[A-Fa-f0-9]{0,4}";
   $pattern7 = "([A-Fa-f0-9]{0,4}:){6}:[A-Fa-f0-9]{0,4}";
   if (preg_match("/^($pattern1)$|^($pattern2)$|^($pattern3)$|^($pattern4)$|^($pattern5)$|^($pattern6)$|^($pattern7)$/", $ipv6)) {
      return true;
   } else {
      return false;
   }
}

// IP in Daten-Datei schreiben
// ----------------------------------------------------------------------------
function write_data($file, $string) {
   if ( ($file == "") || ($string == "") ) {
      // Kein Dateiname oder Inhalt zum Schreiben
      write_log("Write file: Filename/content missing");
      return false;
   } else {
      if ( is_writable(DATA_FOLDER) ) {
         // Ordner beschreibbar
         $write_result = file_put_contents( DATA_FOLDER."/".$file.".ip", $string."\n" );
         if ( $write_result === false ) {
            // Schreiben fehlgeschlagen
            write_log("Write file '".$file.".ip' failed");
            return false;
         } else {
            // Schreiben OK, N Bytes geschrieben
            write_debug("Write file '".$file.".ip' OK");
            return true;
         }
      } else {
         // Ordner nicht beschreibbar
         write_log("Write: Folder '".DATA_FOLDER."' not writable");
         return false;
      }
   }
}

// IP aus Daten-Datei lesen
// ----------------------------------------------------------------------------
function read_data($file) {
   if ( $file == "" ) {
      // Kein Dateiname angegeben
      write_log("Read file: Filename missing");
      return false;
   } else {
      if ( is_readable(DATA_FOLDER."/".$file.".ip") ) {
         // Inhalt lesen
         $read_result = file_get_contents( DATA_FOLDER."/".$file.".ip" );
         write_debug("Read file '".$file.".ip' OK");
         return trim(str_replace("\n","", $read_result));
      } else {
         // Ordner nicht lesbar/vorhanden
         write_log("Read file: File '".$file.".ip' not readable");
         return false;
      }
   }
}

// Zeitstempel
// ----------------------------------------------------------------------------
function simple_timestamp() {
   // 2015-03-04_05:53:20
   $my_timestamp = date("Y-m-d_H:i:s_T");
   return $my_timestamp;
}

// Protokollierung (Ereignisse)
// ----------------------------------------------------------------------------
function write_log($string) {
   $write_result = file_put_contents(LOG_FILE, TIMESTAMP." ".pad_ipv4(CLIENT_IP)." ".$string."\n", FILE_APPEND);
   if ( $write_result === false ) {
      // Schreiben fehlgeschlagen
      return "Error";
   } else {
      // Schreiben OK, N Bytes geschrieben
      return "OK, ".$write_result." bytes written";            
   }
}

// Protokollierung (Debugging)
// ----------------------------------------------------------------------------
function write_debug($string) {
   if ( DEBUG === true ) {
      // Erst normales Logging
      write_log($string);
      // Und wenn Debugging aktiv
      $write_result = file_put_contents(LOG_FILE_DBG, TIMESTAMP." ".pad_ipv4(CLIENT_IP)." ".$string."\n", FILE_APPEND);
      if ( $write_result === false ) {
         // Schreiben fehlgeschlagen
         return "Error";
      } else {
         // Schreiben OK, N Bytes geschrieben
         return "OK, ".$write_result." bytes written";
      }
   }
}

// DNS-Aktualisierung
// ----------------------------------------------------------------------------
function update_dns($host, $ip) {
   if ( ( $host == "" ) || ( $ip == "" ) ) {
      // Hostname oder IP fehlt
      write_log("Nsupdate: Hostname or IP missing");
      return false;
   } else {
      if ( DNSSOFTWARE == "bind" ) {
         // Bind: String für nsupdate konstruieren und ausführen
         // -------------------------------------------------------------------
         // Server, Key und Zone
         $ddns_update_command  = "server ".DNS_SERVER."\n";
         $ddns_update_command .= "key ".KEY_NAME." ".KEY_VALUE."\n";
         $ddns_update_command .= "zone ".DDNS_ZONE.".\n";
         // A, AAAA und TXT entfernen
         $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". A\n";
         $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". AAAA\n";
         $ddns_update_command .= "update delete ".$host.".".DDNS_ZONE.". TXT\n";
         // A (IPv4) / AAAA (IPv6) und TXT hinzufügen
         $ddns_update_command .= "update add ".$host.".".DDNS_ZONE.". ".TTL_SECONDS." IN ".DNS_RTYPE." ".$ip."\n";
         $ddns_update_command .= "update add ".$host.".".DDNS_ZONE.". ".TTL_SECONDS." IN TXT \"".TIMESTAMP."\"\n";
         $ddns_update_command .= "send\n";
         // Debug
         write_debug("Nsupdate running command '".$ddns_update_command."'");
         // Ausführen
         $cmdout = shell_exec("echo \"".$ddns_update_command. "\" | ".NSUPDATE." 2>&1; echo $?");
         // Debug
         write_debug("Nsupdate result '".print_error_status($cmdout)."'");
      }

      if ( DNSSOFTWARE == "dnsmasq" ) {
         // dnsmasq: Dateien schreiben
         // -------------------------------------------------------------------
         // Serial: YYYYMMDDNN
         $zone_serial_base = date('Ymd');
         $current_hour     = date('H');
         $current_minute   = date('i');
         $minute_of_day    = ((60 * $current_hour) + $current_minute);
         $zone_serial_tag  = floor($minute_of_day / 14.41);
         $zone_serial      = $zone_serial_base.$zone_serial_tag;
         $file_timestamp   = date("Ymd")." (".date("H:i").")";

         // Hosts-Datei schreiben
         $file_name        = DNSM_FOLDER.$host.".".DDNS_ZONE.".conf";
         // Inhalt
         $file_host_record  = "# ".$file_timestamp."\n";
         $file_host_record .= "host-record=".$host.".".DDNS_ZONE.",".$ip."\n";
         $write_result = file_put_contents($file_name, $file_host_record);
         if ( $write_result === false ) {
            // Schreiben fehlgeschlagen
            write_debug("Write file '".$host.".".DDNS_ZONE.".conf' failed");
         } else {
            // Schreiben OK, N Bytes geschrieben
            write_debug("Write file '".$host.".".DDNS_ZONE.".conf' OK");
         }

         // SOA - Datei schreiben
         $file_name        = DNSM_FOLDER."soa.conf";
         // Inhalt
         $file_soa_record  = "# ".$file_timestamp."\n";
         $file_soa_record .= "auth-soa=".$zone_serial.",".SOA_RNAME;
         $file_soa_record .= ",".SOA_REFRESH.",".SOA_RETRY.",".SOA_EXPIRY."\n";
         $write_result = file_put_contents($file_name, $file_soa_record);
         if ( $write_result === false ) {
            // Schreiben fehlgeschlagen
            write_debug("Write file 'soa.conf' failed");
         } else {
            // Schreiben OK, N Bytes geschrieben
            write_debug("Write file 'soa.conf' OK");
         }         
      }
      // Ende
      // ----------------------------------------------------------------------
      return true;
   }
}

// Zeitstempel in lesbares Format 
// ----------------------------------------------------------------------------
function timestamp_to_readable($timestamp) {
   // 20150304-055820
   return date("Ymd-His", $timestamp);
}

// Fehler-Anzeige
// ----------------------------------------------------------------------------
function print_error_status($errorcode) {
   $errorcode = trim(str_replace("\n", "", $errorcode));
   if ( $errorcode == "0" ) {
      // Erfolg
      return "OK";
   } else {
      // Fehler
      return "Error '#".$errorcode."'";
   }
}

// IPv4-Adresse füllen auf xxx.xxx.xxx.xxx
// ----------------------------------------------------------------------------
function pad_ipv4($address) {
   // Zerschneiden
   $ip_parts = explode(".", $address);
   // Füllen
   $ip_parts[0] = str_pad($ip_parts[0], 3, "0", STR_PAD_LEFT);
   $ip_parts[1] = str_pad($ip_parts[1], 3, "0", STR_PAD_LEFT);
   $ip_parts[2] = str_pad($ip_parts[2], 3, "0", STR_PAD_LEFT);
   $ip_parts[3] = str_pad($ip_parts[3], 3, "0", STR_PAD_LEFT);
   // Zusammenkleben
   $address = implode(".", $ip_parts);
   // Return
   return $address;
}

// Bool zu String
// ----------------------------------------------------------------------------
function strbool($value) {
   return $value ? 'OK' : 'Not OK';
}
?>
